@@ -1,4 +1,4 @@
-from . import util
+from . import util, valid
 from .exceptions import KrankshaftError
 from .query import DjangoQuery
 from django.core.files import File
@@ -10,6 +10,13 @@ try:
 except ImportError:
     from .compat.xact import xact as atomic
 
+# TODO implement some concepts from:
+#   http://clevertech.biz/devblog/successful-rest-api-design/
+#
+#   - metadata follow-up actions
+#
+# TODO also: http://stateless.co/hal_specification.html
+#
 # TODO consider subresources instead of prefetch_related()
 #   ie: manytomany: [
 #       '/api/v1/many/1/',
@@ -29,6 +36,11 @@ except ImportError:
 #   - current behavior of prefetch_related() and resource uris to related
 #   - subresource behavior described above
 #
+# new thought: /api/v1/resource/set/1;2;3/many/
+# considerably easier to construct and saves the api from expensive
+# prefetch_related just to expose related urls... This probably needs to just
+# happen...
+#
 # XXX both methods have pros and cons
 #   current:
 #       pros:
@@ -48,6 +60,7 @@ except ImportError:
 #             for all related resources... its non-trivial
 #             (does object and related get merged on the server or client?
 #             it may only be possible on the server...)
+#             TODO SOLVED!!!
 #
 # regardless, at this point, both methods are trivial to implement...
 class DjangoModelResource(object):
@@ -249,7 +262,7 @@ class DjangoModelResource(object):
         code = None
         detail = None
         while True:
-            if self.version_field and instance:
+            if instance and instance.pk and self.version_field:
                 if self.version_field not in clean:
                     code = 400
                     detail = {
@@ -331,7 +344,7 @@ class DjangoModelResource(object):
         instances.sort(key=lambda instance: ids.index(instance.pk))
         return instances
 
-    def fetch_list(self, request, query):
+    def fetch_list(self, request, query, qs=None):
         '''fetch_list(request, query) -> instances
 
         Ideally, calling functions would do:
@@ -343,8 +356,11 @@ class DjangoModelResource(object):
         if not isinstance(query, self.Query):
             query = self.make_query(query)
 
+        if qs is None:
+            qs = self.get_query_set(request)
+
         try:
-            qs, meta = query.apply(self.get_query_set(request))
+            qs, meta = query.apply(qs)
         except self.Query.Issues as exc:
             self.api.abort(self.api.serialize(request, 403, {
                 'error': 'There are issues with your query',
@@ -364,14 +380,17 @@ class DjangoModelResource(object):
 
         return instances, meta
 
-    def fetch_set(self, request, idset):
+    def fetch_set(self, request, idset, qs=None):
         '''fetch_set(request, idset) -> instances
 
         Fetch a set of instances given idset which is in the format of a string
         of semi-colon separated ids.  Handles ensuring all instances are found.
         '''
+        if qs is None:
+            qs = self.get_query_set(request)
+
         idset = [self.clean_id(request, id) for id in idset.split(';')]
-        instances = list(self.get_query_set(request).filter(pk__in=idset))
+        instances = list(qs.filter(pk__in=idset))
 
         set_ids = set(idset)
         set_instance_pks = set([instance.pk for instance in instances])
@@ -386,7 +405,7 @@ class DjangoModelResource(object):
 
         return instances
 
-    def fetch_single(self, request, query, id):
+    def fetch_single(self, request, query, id, qs=None):
         '''fetch_single(request, query, id) -> instance
 
         Fetch a single instance from the database handling when an instance
@@ -395,7 +414,9 @@ class DjangoModelResource(object):
         if query and not isinstance(query, self.Query):
             query = self.make_query(query)
 
-        qs = self.get_query_set(request)
+        if qs is None:
+            qs = self.get_query_set(request)
+
         meta = None
         if query:
             try:
@@ -497,11 +518,16 @@ class DjangoModelResource(object):
 
     def make_related_validator(self, field, expected, many=False):
         def related_validator(value, expect):
+            if value:
+                try:
+                    value = expect([valid.string], value if many else [value])
+                except expect.ValueIssue:
+                    raise ValueError('Expected uri')
+
             resource = self.related_lookup(field)
             if value is not None and resource:
                 try:
-                    resolved_resource, value = \
-                        self.api.resolve(value if many else [value])
+                    resolved_resource, value = self.api.resolve(value)
 
                     if resolved_resource != resource:
                         raise self.api.ResolveError(
@@ -749,6 +775,8 @@ class DjangoModelResource(object):
                 else:
                     data[field.name] = getattr(instance, field.name)
 
+        data['_id'] = instance.pk
+        data['_pk'] = self.pk_name
         data['_uri'] = self.reverse_single(instance.pk)
 
         return data
@@ -782,7 +810,8 @@ class DjangoModelResource(object):
             else:
                 if isinstance(field, models.ManyToManyField):
                     resource = self.related_lookup(field)
-                    manytomany[field] = resource.fetch(*clean[field.name])
+                    if resource:
+                        manytomany[field] = resource.fetch(*clean[field.name])
 
                 elif isinstance(field, models.ForeignKey):
                     setattr(instance, field.name + '_id', clean[field.name])
@@ -1079,8 +1108,7 @@ class DjangoModelResource(object):
 
             for obj in clean:
                 id = obj[self.pk_name]
-                instance = instance_lookup[id]
-                self.deserialize(request, obj, instance)
+                self.update(instance_lookup[id], obj)
 
         if self.use_location:
             return self.api.response(request, 204,
